




def add_paths_to_system (paths):
	import pathlib
	from os.path import dirname, join, normpath
	import sys
	
	this_folder = pathlib.Path (__file__).parent.resolve ()	
	for path in paths:
		sys.path.insert (0, normpath (join (this_folder, path)))

add_paths_to_system ([
	'../../../modules',
	'../../../modules_pip'
])


import json
import pathlib
from os.path import dirname, join, normpath


this_module_name = ""

this_folder = pathlib.Path (__file__).parent.resolve ()
structures = str (normpath (join (this_folder, "../../../../structures")))

this_module = str (normpath (join (structures, "modules", this_module_name)))

#status_assurances_path = str (normpath (join (this_folder, "insurance")))
status_assurances_path = str (normpath (join (this_folder, "..")))

import sys
if (len (sys.argv) >= 2):
	glob_string = status_assurances_path + '/' + sys.argv [1]
	db_directory = False
else:
	glob_string = status_assurances_path + '/**/status_*.py'
	db_directory = normpath (join (this_folder, "DB"))

print ("glob string:", glob_string)

import onesie
scan = onesie.start (
	glob_string = glob_string,
	simultaneous = True,
	module_paths = [
		normpath (join (structures, "modules")),
		normpath (join (structures, "modules_pip"))
	],
	relative_path = status_assurances_path,
	
	db_directory = db_directory
)




'''
	python3 status.proc.py "status_2.py"
'''

import law_dictionary

def check_1 ():
	dictionary = {}

	def retrieve_directory ():
		return "/"

	report = law_dictionary.check (	
		return_obstacle_if_not_legit = True,
		laws = {
			"directory_1": {
				"required": False,
				"contingency": retrieve_directory,
				"type": str
			},
			"directory_2": {
				"required": False,
				"contingency": "/drives",
				"type": str
			}
		},
		dictionary = dictionary
	)
	if (report ["advance"] != True):
		raise Exception (report ["obstacle"])	

	assert (dictionary ["directory_1"] == "/")
	assert (dictionary ["directory_2"] == "/drives")

def check_2 ():
	dictionaries = [{ "directory_1": "/" }, { "directory_1": "/drives"  }]

	for dictionary in dictionaries:
		law_dictionary.check (	
			laws = {
				"directory_1": {
					"required": True,
					"allow": [ "/", "/drives" ]
				}
			},
			dictionary = dictionary
		)

def check_3 ():
	dictionary = { 
		"paths": {
			"directory_1": "/" 
		}
	}	
		
	law_dictionary.check (	
		laws = {
			"paths": {
				"required": True,
				"type": dict
			}
		},
		dictionary = dictionary
	)

	law_dictionary.check (	
		laws = {
			"directory_1": {
				"required": True,
				"type": str
			}
		},
		dictionary = dictionary ["paths"]
	)

checks = {
	'check 1': check_1,
	'check 2': check_2,
	'check 3: multiple levels': check_3
}



'''
	python3 status.proc.py "status_broken_1.py"
'''

import law_dictionary

def broken_1 ():
	def obstacle (prob):
		return prob


	dictionary = {}
	the_obstacle = law_dictionary.check (	
		laws = {
			"directory": {
				"required": True,
				"contingency": "/",
				"allow": [ "/" ]
			}
		},
		dictionary = dictionary,
		obstacle = obstacle		
	)
	
	print ("the obstacle:", the_obstacle)
	
	assert (
		the_obstacle == 
		'The label "directory" was not found in the laws.'
	)
	

def broken_2 ():
	def obstacle (prob):
		return prob

	dictionary = {
		"directory": "/",
		"directory 2": "/"
	}
	
	the_obstacle = law_dictionary.check (	
		laws = {
			"directory": {
				"required": True,				
				"allow": [ "" ]
			},
			"directory 2": {
				"required": True,
				"allow": [ "/" ]
			}
		},
		dictionary = dictionary,
		
		obstacle = obstacle
	)
	
	print ("the obstacle:", the_obstacle)
	
	assert (
		the_obstacle == 
		'Defintion "/" is not allowed.'
	)
	

checks = {
	'broken 1': broken_1,
	'broken 2': broken_2
}





'''
	python3 status.proc.py "status_broken_2.py"
'''

import law_dictionary

def broken_1 ():
	dictionary = {
		"directory_1": "/",
		"directory_2": "/drives"
	}
	report = law_dictionary.check (
		return_obstacle_if_not_legit = True,
		allow_extra_fields = False,
		laws = {
			"directory_1": {
				"required": True
			}
		},
		dictionary = dictionary
	)
	if (report ["advance"] != True):
		pass;
	
	assert (report ["advance"] == False), report
	assert (report ["obstacle"] == 'The label "directory_2" was found in the dictionary but not in the laws.'), report


checks = {
	'broken 1': broken_1
}





'''
	{
		dictionary_health,
		dictionary_trends,
		law_dictionary,
		dictionary_customs,
		dictator,
		tater,
		potatoe,
		potato salad,
		vegan_butter
	}
'''

'''
	import solitaire.modules.law_dictionary as law_dictionary

	#
	#	contigencies
	#
	law_dictionary.check (
		allow_extra_fields = False,
		
		laws = {
			"directory": {
				"required": True,
				
				#
				#	if "required" == False and does not have "directory",
				#	then either:
				#		( ) use the value provided
				#		( ) call the function
				#
				"contingency": retrieve_directory
			}
		},
		dictionary = {}
	)

	def retrieve_directory ():
		return os.path.dirname (
			os.path.abspath ((inspect.stack ()[1]) [1])
		)

	law_dictionary.check (	
		laws = {
			"directory": {
				"required": false,
				"contingency": retrieve_directory,
				"type": str
			}
		},
		dictionary = {}
	)
'''

import rich
import types

def obstacle (string):
	raise Exception (string)
	


def check (
	allow_extra_fields = False,
	return_obstacle_if_not_legit = False,
	
	laws = {},
	dictionary = {},
	
	obstacle = obstacle
):
	if (return_obstacle_if_not_legit == True):
		def obstacle_activity (prob):
			return {
				"advance": False,
				"obstacle": prob
			}
		
		obstacle = obstacle_activity

	if (allow_extra_fields != True):
		for label in dictionary:
			if (label not in laws):
				return obstacle (f'The label "{ label }" was found in the dictionary but not in the laws.')
			
	for law_label in laws:	
		law_label_params = laws [ law_label ]
	
		if ("required" in law_label_params):
			required = law_label_params [ "required" ]
		else:
			required = True
	
		if (required == True):
			if (law_label not in dictionary):
				return obstacle (f'The label "{ law_label }" was not found in the laws.')
			
		else:	
			if ("contingency" in law_label_params):
				contingency = law_label_params [ "contingency" ]
				#print ('has contingency', contingency)
				
				if (type (contingency) == types.FunctionType):
					dictionary [ law_label ] = contingency ();
				else:
					dictionary [ law_label ] = contingency;

			else:
				return obstacle (
					''.join ([
						f'Non-required laws must have a contingency.',
						f'"{ law_label }" does not have a contingency'
					])
				)

		'''
			After the required piece, the definition can be
			100% determined.
		'''
		dictionary_definition = dictionary [ law_label ];
		
		if ("type" in law_label_params):
			label_type = law_label_params [ "type" ]
			
			if (type (dictionary [ law_label ]) != label_type):
				return obstacle (f'Defintion "{ dictionary [ law_label ] }" is not type "{ label_type }".')
		
		
		if ("allow" in law_label_params):
			label_allow = law_label_params [ "allow" ]
		
			if (type (label_allow) == types.FunctionType):
				allowed = label_allow ();
			else:
				allowed = label_allow
				
			if (dictionary_definition not in allowed):
				return obstacle (f'Defintion "{ dictionary_definition }" is not allowed.')
		
		#print (law_label_params)
		
	return {
		"advance": True
	}